#include <Wire.h>
#include <Adafruit_PWMServoDriver.h>
#include <Servo.h>
#include <ESP8266WiFi.h>


const char* ssid = "";
const char* password = "";

Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

#define SERVOMIN  150 // Minimum pulse length count (for 0 degrees)
#define SERVOMAX  570 // Maximum pulse length count (for 180 degrees)
#define SERVO_FREQ 50  // Analog servos run at ~50 Hz

Servo servoC0;  // Servo object for C0
Servo servoC1;  // Servo object for C1
Servo servoC2;  // Servo object for C2

WiFiServer server(80);

// Function to set the angle of the servo on a given channel
void setServoAngle(uint8_t channel, uint16_t angle) {
  uint16_t pulse = map(angle, 0, 180, SERVOMIN, SERVOMAX);

  if (channel == 0) {
    servoC0.write(angle);  // Set C0 (attached to D7) to specified angle
  } else if (channel == 1) {
    servoC1.write(angle);  // Set C1 (attached to D6) to specified angle
  } else if (channel == 2) {
    servoC2.write(angle);  // Set C2 (attached to D5) to specified angle
  } else {
    pwm.setPWM(channel, 0, pulse);  // Set angle for channels C3 - C15 using PWM
  }
}

void setup() {
  Serial.begin(115200);
         // Initialize serial communication


  // Initialize PWM driver
  pwm.begin();
  pwm.setPWMFreq(SERVO_FREQ);
  delay(10);

  // Attach Servo objects to their respective pins
  servoC0.attach(D7);  // Attach C0 to digital pin D7
  servoC1.attach(D6);  // Attach C1 to digital pin D6
  servoC2.attach(D5);  // Attach C2 to digital pin D5 (corrected missing semicolon)

  // Connect to WiFi
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
    Serial.println("Connecting to WiFi...");
  }
  Serial.println("Connected to WiFi");
  Serial.print("Access the control panel at: ");
  Serial.println(WiFi.localIP());

  // Start the server
  server.begin();
}

void loop() {
  

  // Send a 10µs HIGH pulse to the Trig pin
 

  // Measure the pulse duration on the Echo pin
  

  // Calculate the distance in cm
  

  // Print the distance to the Serial Monitor
 

  // Check for client connection
  WiFiClient client = server.available();  // Ensure 'server' is defined elsewhere

  if (client) {
    while (client.connected()) {
      if (client.available()) {
        String input = client.readStringUntil('\n'); // Read the input from client

        // Parse channel and angle
        int channel = -1;
        int angle = -1;

        if (input[0] == 'C' && input.indexOf(':') != -1) {
          channel = input.substring(1, input.indexOf(':')).toInt();
          angle = input.substring(input.indexOf(':') + 1).toInt();
        }

        // Validate and execute command
        if (channel >= 0 && channel <= 15 && angle >= 0 && angle <= 180) {
          setServoAngle(channel, angle);  // Ensure 'setServoAngle' is defined elsewhere
          Serial.print("Moved servo on channel ");
          Serial.print(channel);
          Serial.print(" to ");
          Serial.print(angle);
          Serial.println(" degrees.");
          client.println("Servo moved successfully.");
        } else {
          Serial.println("Invalid input format. Use format C#:Angle (e.g., C4:90 for Channel 4 at 90 degrees)");
          client.println("Error: Invalid input format. Use format C#:Angle.");
        }
      }
    }
    client.stop();
  }
}

